
import pytest

import ggps


def expected_tcm_first_trackpoint():
    return {
        "altitudefeet": "850.3937408367167",
        "altitudemeters": "259.20001220703125",
        "distancekilometers": "0.0",
        "distancemeters": "0.0",
        "distancemiles": "0.0",
        "elapsedtime": "00:00:00",
        "heartratebpm": "85",
        "latitudedegrees": "44.97431952506304",
        "longitudedegrees": "-93.26310088858008",
        "cadence": "89",
        "cadencex2": "178",
        "seq": "1",
        "speed": "0.0",
        "time": "2014-10-05T13:07:53.000Z",
        "type": "Trackpoint"
    }

def expected_tcm_middle_trackpoint():
    return {
        "altitudefeet": "805.7742982398804",
        "altitudemeters": "245.60000610351562",
        "distancekilometers": "32.187189453125",
        "distancemeters": "32187.189453125",
        "distancemiles": "20.00019228525722",
        "elapsedtime": "03:13:19",
        "heartratebpm": "140",
        "latitudedegrees": "44.959017438814044",
        "longitudedegrees": "-93.21290854364634",
        "cadence": "84",
        "cadencex2": "168",
        "seq": "1747",
        "speed": "2.8269999027252193",
        "time": "2014-10-05T16:21:12.000Z",
        "type": "Trackpoint"
    }

def expected_tcm_last_trackpoint():
    return {
        "altitudefeet": "864.8294163501167",
        "altitudemeters": "263.6000061035156",
        "distancekilometers": "42.63544921875",
        "distancemeters": "42635.44921875",
        "distancemiles": "26.492439912628992",
        "elapsedtime": "04:14:24",
        "heartratebpm": "161",
        "latitudedegrees": "44.95180849917233",
        "longitudedegrees": "-93.10493202880025",
        "cadence": "77",
        "cadencex2": "154",
        "seq": "2256",
        "speed": "3.5460000038146977",
        "time": "2014-10-05T17:22:17.000Z",
        "type": "Trackpoint"
    }

def test_twin_cities_marathon_tcx_file():
    filename = 'data/twin_cities_marathon.tcx'
    handler = ggps.TcxHandler()
    handler.parse(filename)

    tkpts = handler.trackpoints
    expected_tcm_attr_count = 15

    assert(handler.curr_depth() == 0)
    assert(handler.curr_path() == '')

    # check the number of trackpoints
    actual = len(tkpts)
    expected = 2256
    assert(actual == expected)
    actual = handler.trackpoint_count()
    assert(actual == expected)

    # check the first trackpoint
    expected_tkpt = expected_tcm_first_trackpoint()
    expected_keys = sorted(expected_tkpt.keys())
    actual_tkpt   = handler.trackpoints[0]
    actual_keys   = sorted(actual_tkpt.values.keys())
    assert(expected_keys == actual_keys)

    assert(len(actual_tkpt.values) == expected_tcm_attr_count)
    for key in expected_tkpt.keys():
        expected, actual = expected_tkpt[key], actual_tkpt.values[key]
        assert(expected == actual)

    # check a trackpoint at ~mile 20
    expected_tcm_tkpt = expected_tcm_middle_trackpoint()
    actual_tkpt = handler.trackpoints[1746]
    assert(len(actual_tkpt.values) == expected_tcm_attr_count)
    for key in expected_tcm_tkpt.keys():
        expected, actual = expected_tcm_tkpt[key], actual_tkpt.values[key]
        assert(expected == actual)

    # check the last trackpoint
    expected_tcm_tkpt = expected_tcm_last_trackpoint()
    actual_tkpt = handler.trackpoints[-1]
    assert(len(actual_tkpt.values) == expected_tcm_attr_count)
    for key in expected_tcm_tkpt.keys():
        expected, actual = expected_tcm_tkpt[key], actual_tkpt.values[key]
        assert(expected == actual)

    # check seconds_to_midnight
    secs = int(handler.first_time_secs_to_midnight)
    assert(secs > 0)
    assert(secs < 86400)

def test_dc_trails_20200217_tcx_file():
    filename = 'data/activity_4564516081.tcx'
    handler = ggps.TcxHandler()
    handler.parse(filename)
    tkpts = handler.trackpoints

    # check the number of trackpoints
    actual = len(tkpts)
    expected = 2209
    assert(actual == expected)





import json
import pytest

import ggps


def test_str():
    handler = ggps.PathHandler()
    actual = str(handler)
    expected = '{}'
    assert(actual == expected)

    filename = 'data/twin_cities_marathon.gpx'
    handler = ggps.PathHandler()
    handler.parse(filename)
    obj = json.loads(str(handler))
    cnt = obj['gpx|trk|trkseg|trkpt@lat']
    assert(cnt == 2256)

def test_counts():
    filename = 'data/twin_cities_marathon.gpx'
    handler = ggps.PathHandler()
    handler.parse(filename)
    counter = handler.path_counter
    cnt = counter['gpx|trk|trkseg|trkpt@lat']
    assert(cnt == 2256)
    cnt = counter['gpx|metadata|time']
    assert(cnt == 1)

def test_base_parse_hhmmss():
    filename = 'data/twin_cities_marathon.gpx'
    handler = ggps.PathHandler()
    handler.parse(filename)
    hhmmss = handler.parse_hhmmss('')
    assert(hhmmss == '')
    hhmmss = handler.parse_hhmmss('xxx')
    assert(hhmmss == '')




import inspect
import json
import sys
import time
import os

import ggps

# python ggps-examples.py > data/ggps-examples.txt


if __name__ == "__main__":

    print('version {}'.format(ggps.VERSION))
    print(str(inspect.ismodule(ggps)))

    if True:
        infile = 'data/twin_cities_marathon.tcx'
        handler = ggps.PathHandler()
        handler.parse(infile)
        print(str(handler))
        obj = json.loads(str(handler))

    if True:
        infile = 'data/twin_cities_marathon.tcx'
        handler = ggps.TcxHandler()
        handler.parse(infile)
        trackpoints = handler.trackpoints
        count = len(trackpoints)
        print('{} trackpoints loaded from file {}'.format(count, infile))
        for t in trackpoints:
            print(repr(t))
        print('---')
        print(repr(trackpoints[-1]))
        print('{} trackpoints loaded from file {}'.format(count, infile))

    if True:
        infile = 'data/twin_cities_marathon.gpx'
        handler = ggps.GpxHandler()
        handler.parse(infile)
        trackpoints = handler.trackpoints
        count = len(trackpoints)
        print('{} trackpoints loaded from file {}'.format(count, infile))
        for t in trackpoints:
            print(repr(t))
        print('---')
        print(repr(trackpoints[-1]))
        print('{} trackpoints loaded from file {}'.format(count, infile))

    if True:
        infile = 'data/activity_4564516081.tcx'
        handler = ggps.TcxHandler()
        handler.parse(infile)
        trackpoints = handler.trackpoints
        count = len(trackpoints)
        print('{} trackpoints loaded from file {}'.format(count, infile))
        for t in trackpoints:
            print(repr(t))
        print('trackpoint count: {}'.format(len(trackpoints)))

    if False:
        infile = 'data/activity_4564516081.gpx'
        handler = ggps.GpxHandler()
        handler.parse(infile)
        trackpoints = handler.trackpoints
        count = len(trackpoints)
        print('{} trackpoints loaded from file {}'.format(count, infile))
        for t in trackpoints:
            print(repr(t))

    print('version {}'.format(ggps.VERSION))
    print(str(inspect.ismodule(ggps)))





import xml.sax

import m26

from ggps.trackpoint import Trackpoint


class BaseHandler(xml.sax.ContentHandler):

    def __init__(self):
        xml.sax.ContentHandler.__init__(self)
        self.heirarchy = list()
        self.trackpoints = list()
        self.curr_tkpt = Trackpoint()
        self.curr_text = ''
        self.end_reached = False
        self.first_time = None
        self.first_etime = None
        self.first_time_secs_to_midnight = 0

    def endDocument(self):
        self.completed = True

    def characters(self, chars):
        if self.curr_text:
            self.curr_text = self.curr_text + chars
        else:
            self.curr_text = chars

    def reset_curr_text(self):
        self.curr_text = ''

    def curr_depth(self):
        return len(self.heirarchy)

    def curr_path(self):
        return '|'.join(self.heirarchy)

    def trackpoint_count(self):
        return len(self.trackpoints)

    def set_first_trackpoint(self, t):
        self.first_time = t.get('time')
        self.first_hhmmss = self.parse_hhmmss(self.first_time)
        self.first_etime = m26.ElapsedTime(self.first_hhmmss)
        self.first_time_secs = self.first_etime.secs

        # deal with the possibility that the Activity spans two calendar days.
        secs = int(m26.Constants.seconds_per_hour() * 24)
        self.first_time_secs_to_midnight = secs - self.first_time_secs
        if False:
            print("first_time:   {0}".format(self.first_time))
            print("first_hhmmss: {0}".format(self.first_hhmmss))
            print("first_etime:  {0}".format(self.first_etime))
            print("first_time_secs: {0}".format(self.first_time_secs))
            print("first_time_secs_to_midnight: {0} {1}".format(
                self.first_time_secs_to_midnight, secs))

    def meters_to_feet(self, t, meters_key, new_key):
        m = t.get(meters_key)
        km = float(m) / 1000.0
        d_km = m26.Distance(km, m26.Constants.uom_kilometers())
        yds = d_km.as_yards()
        t.set(new_key, str(yds * 3.000000))

    def meters_to_km(self, t, meters_key, new_key):
        m = t.get(meters_key)
        km = float(m) / 1000.0
        t.set(new_key, str(km))

    def meters_to_miles(self, t, meters_key, new_key):
        m = t.get(meters_key)
        km = float(m) / 1000.0
        d_km = m26.Distance(km, m26.Constants.uom_kilometers())
        t.set(new_key, str(d_km.as_miles()))

    def cadence_x2(self, t):
        c = t.get('cadence', 0)
        i = int(c)
        if i > 0:
            t.set('cadencex2', str(i * 2))

    def calculate_elapsed_time(self, t):
        time_str = t.get('time')
        new_key = 'elapsedtime'
        if time_str:
            if time_str == self.first_time:
                t.set(new_key, '00:00:00')
            else:
                curr_time = self.parse_hhmmss(time_str)
                curr_etime = m26.ElapsedTime(curr_time.strip())
                secs_diff = curr_etime.secs - self.first_time_secs
                if secs_diff < 0:
                    secs_diff = secs_diff + self.first_time_secs_to_midnight
                elapsed = m26.ElapsedTime(secs_diff)
                t.set(new_key, elapsed.as_hhmmss())

    def parse_hhmmss(self, time_str):
        """
        For a given datetime value like '2014-10-05T17:22:17.000Z' return the
        hhmmss value '17:22:17'.
        """
        if len(time_str) > 0:
            if 'T' in time_str:
                return str(time_str.split('T')[1][:8])
            else:
                return ''
        else:
            return ''




import json
import pytest

import ggps


def test_str():
    t = ggps.Trackpoint()
    assert(str(t) == '<Trackpoint values count:1>')

def test_repr():
    t = ggps.Trackpoint()
    t.set('lang', 'python')
    t.set(None, 'python')
    j = repr(t)
    obj = json.loads(j)
    assert(obj['type'] == 'Trackpoint')
    assert(obj['lang'] == 'python')

def test_get():
    t = ggps.Trackpoint()
    t.set('lang', 'python')
    assert(t.get('lang') == 'python')
    assert(t.get('xxx') == '')
    assert(t.get('zzz', 'zero') == 'zero')



from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ggps',

    # Versions should comply with PEP440.
    version='0.3.0',

    description='ggps is a python library for parsing and enhancing Garmin gpx and tcx files',
    long_description=long_description,

    url='https://github.com/cjoakim/ggps',

    # Author details
    author='Christopher Joakim',
    author_email='christopher.joakim@gmail.com',

    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',
        'Topic :: Utilities',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    keywords='garmin gps devices gpx tcx',

    # You can just specify the packages manually here if your project is simple.
    # Or you can use find_packages().
    packages=find_packages(exclude=['bin', 'contrib', 'htmlcov', 'data', 'docs', 'tests*']),

    # Run-time dependencies; these will be installed by pip when the project is installed.
    # See https://packaging.python.org/en/latest/requirements.html
    install_requires=['m26'],

    # Dev and Test dependencies, install like this: $ pip install -e .[dev,test]
    extras_require={
        'dev':  ['check-manifest', 'flake8'],
        'test': ['coverage', 'pytest', 'pytest-cov'],
    },
)



__author__ = 'cjoakim'
__version__ = '0.3.0'

"""
ggps library
"""

VERSION = __version__

from ggps.trackpoint import Trackpoint
from ggps.base_handler import BaseHandler
from ggps.gpx_handler import GpxHandler
from ggps.path_handler import PathHandler
from ggps.tcx_handler import TcxHandler
